package br.ime.usp.mac5855.otimizacao.exemplos;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;

import br.ime.usp.mac5855.otimizacao.Dominio;
import br.ime.usp.mac5855.otimizacao.FuncaoDeCusto;
import br.ime.usp.mac5855.otimizacao.Otimizador;
import br.ime.usp.mac5855.otimizacao.Resultado;

public class DormitorioDeEstudantes implements FuncaoDeCusto {

	private String [] dormitorios = new String[]{"Zeus", "Athena","Hercules","Bacchus","Pluto"};
	
	private List<String[]> preferencias;
	
	public DormitorioDeEstudantes(){
		preferencias = getListaDePreferencias();
	}
	
	private static List<String[]> getListaDePreferencias() {
		InputStream is = DormitorioDeEstudantes.class.getResourceAsStream("/resource/dormprefs.txt");
		Scanner reader = new Scanner(is);
		List<String[]> lista = new ArrayList<String[]>();
		while(reader.hasNextLine()){
			String linha = reader.nextLine();
			lista.add(linha.split(","));
			
		}			
		return lista;
	}

	public void imprimeAlunosEDormitorios(int ... solucao){
		List<Integer> slots = criaSlots();
		for(int i = 0; i < solucao.length; i++){
			int x = solucao[i];
			String dormitorio = dormitorios[slots.get(x)];
			System.out.println(preferencias.get(i)[0] + " " + dormitorio);
			slots.remove(x);
		}
		
	}

	private List<Integer> criaSlots() {
		List<Integer> slots = new ArrayList<Integer>();
		for(int i = 0; i < dormitorios.length; i++){
			slots.add(i);
			slots.add(i);
		}
		return slots;
	}
	
	public double calculaCusto(int... solucao) {
		int custo = 0;
		List<Integer> slots = criaSlots();
		for(int i = 0; i < solucao.length; i++){
			int x = solucao[i];
			String dormitorio = dormitorios[slots.get(x)];
			String [] preferencia = preferencias.get(i);
			if(preferencia[0].equals(dormitorio)){
				custo += 0;
			}else if(preferencia[1].equals(dormitorio)){
				custo += 1;
			}else {
				custo += 3;
			}
			slots.remove(x);
		}
		
		return custo;
	}

	private int getQuantidadeDormitorios() {
		return dormitorios.length;
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		DormitorioDeEstudantes dormitorios = new DormitorioDeEstudantes();
		Otimizador otimizador = new Otimizador();
		Dominio dominio = new Dominio();
		int limite = dormitorios.getQuantidadeDormitorios() * 2;
		for(int i = 0; i < limite; i++){
			dominio.adiciona(0, limite - i - 1);
		}
		System.out.println("Otimizacao randomica:");
		Resultado resultado = otimizador.otimizacaoRandomica(dominio, dormitorios);
		System.out.println(resultado.getValor());
		dormitorios.imprimeAlunosEDormitorios(resultado.getSolucao());
		
		System.out.println("\nOtimizacao por simulacao de aquecimento e esfriamento:");
		resultado = otimizador.otimizacaoPorSimulacaoDeAquecimentoEsfriamento(dominio, dormitorios);
		System.out.println(resultado.getValor());
		dormitorios.imprimeAlunosEDormitorios(resultado.getSolucao());
		
		System.out.println("\nOtimizacao genetica:");
		resultado = otimizador.otimizacaoGenetica(dominio, dormitorios);
		System.out.println(resultado.getValor());
		dormitorios.imprimeAlunosEDormitorios(resultado.getSolucao());
		
		System.out.println("\nOtimizacao gulosa:");
		resultado = otimizador.otimizacaoGulosa(dominio, dormitorios);
		System.out.println(resultado.getValor());
		dormitorios.imprimeAlunosEDormitorios(resultado.getSolucao());
		
	}

	

}
