package br.ime.usp.mac5855.otimizacao.exemplos;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Scanner;

import br.ime.usp.mac5855.otimizacao.Dominio;
import br.ime.usp.mac5855.otimizacao.FuncaoDeCusto;
import br.ime.usp.mac5855.otimizacao.Otimizador;
import br.ime.usp.mac5855.otimizacao.Resultado;

public class ViagemEmGrupo implements FuncaoDeCusto {

	private String [][] pessoasEOrigens = new String[][]{
											{"Seymour","BOS"},
											{"Franny" ,"DAL"},
											{"Zooey"  ,"CAK"},
											{"Walt"   ,"MIA"},
											{"Buddy"  ,"ORD"},
											{"Les"    ,"OMA"},
								};
		
	private Map<String, List<Voo>> voos = null;
	
	private String destino = "LGA";
	
	public ViagemEmGrupo(){
		this.voos = getListaDeVoos();
	}
	
	public double calculaCusto(int... solucao) {
		double precoTotal = 0.0;
		long ultimaChegada = 0;
		long primeiraSaida = 24 * 60; 
		
		for(int i = 0; i < solucao.length / 2; i++){
			String origem = pessoasEOrigens[i][1];
			Voo ida = voos.get(origem+destino).get(solucao[i]);
			Voo volta = voos.get(origem+destino).get(solucao[i+1]);
			precoTotal += (ida.preco + volta.preco);
			
			if(ultimaChegada < ida.horaDeChegada){
				ultimaChegada = ida.horaDeChegada;
			}
			if(primeiraSaida > volta.horaDePartida){
				primeiraSaida = volta.horaDePartida;
			}
		}
		
		long esperaTotal = 0;
		for(int i = 0; i < solucao.length / 2; i++){
			String origem = pessoasEOrigens[i][1];
			Voo ida = voos.get(origem+destino).get(solucao[i]);
			Voo volta = voos.get(origem+destino).get(solucao[i+1]);
			esperaTotal += (ultimaChegada - ida.horaDePartida);
			esperaTotal += (volta.horaDeChegada - primeiraSaida);
			/* dia a mais para carro extra */
			if(ultimaChegada > primeiraSaida){
				precoTotal += 50; // custo fixo de $50 por dia a mais de aluguel do carro.
			}
		}
		return precoTotal + esperaTotal;
	}

	public void imprimeHorarios(int ... solucao){
		for(int i = 0; i < (solucao.length / 2); i++){
			String nome = pessoasEOrigens[i][0];
			String origem = pessoasEOrigens[i][1];
			Voo ida = voos.get(origem+destino).get(solucao[i]);
			Voo volta = voos.get(destino+origem).get(solucao[i+1]);
			System.out.printf("%10s %10s %5s-%5s $%.2f %5s-%5s $%.2f\n", nome, origem, 
														   ida.getHoraDePartida(), ida.getHoraDeChegada(), ida.preco,
														   volta.getHoraDePartida(), volta.getHoraDeChegada(), volta.preco);
		}
	}
	
	private static Map<String, List<Voo>> getListaDeVoos() {
		InputStream is = Voo.class.getResourceAsStream("/resource/schedule.txt");
		Scanner reader = new Scanner(is);
		Map<String, List<Voo>> mapa = new HashMap<String, List<Voo>>();
		while(reader.hasNextLine()){
			String linha = reader.nextLine();
			String [] split = linha.split(",");
			Voo voo = new Voo(split[0], split[1], Voo.parse(split[2]), Voo.parse(split[3]), Double.parseDouble(split[4]));
			String key = voo.origem + voo.destino;
			if(!mapa.containsKey(key)){
				mapa.put(key, new ArrayList<Voo>());
			}
			mapa.get(key).add(voo);
		}			
		return mapa;
	}
	
	private static class Voo {
		private String origem;
		
		private String destino;
		
		private long horaDePartida;
		
		private long horaDeChegada;
		
		private double preco;
		
		Voo(String origem, String destino, long horaDePartida, long horaDeChegada, double preco){
			this.origem = origem;
			this.destino = destino;
			this.horaDeChegada = horaDeChegada;
			this.horaDePartida = horaDePartida;
			this.preco = preco;
		}
			

		public String getHoraDeChegada() {
			long hora = horaDeChegada / 60;
			long minuto = horaDeChegada % 60;
			return hora + ":" + (minuto < 10 ? "0"+minuto : minuto);
		}


		public Object getHoraDePartida() {
			long hora = horaDePartida / 60;
			long minuto = horaDePartida % 60;
			return hora + ":" + (minuto < 10 ? "0"+minuto : minuto);
		}


		private static long parse(String string) {
			String [] split = string.split(":");
			return Long.parseLong(split[0]) * 60 + Long.parseLong(split[1]);
		}
	}

	public static void main(String[] args) {
		ViagemEmGrupo viagem = new ViagemEmGrupo();
		Dominio dominio = new Dominio();
		Otimizador otimizador = new Otimizador();
		for(int i = 0; i < 10; i++){
			dominio.adiciona(0,8);
		}
		System.out.println("Otimizacao randomica:");
		Resultado resultado = otimizador.otimizacaoRandomica(dominio, viagem);
		System.out.println(resultado.getValor());
		viagem.imprimeHorarios(resultado.getSolucao());
		
		System.out.println("\nOtimizacao por simulacao de aquecimento e esfriamento:");
		resultado = otimizador.otimizacaoPorSimulacaoDeAquecimentoEsfriamento(dominio, viagem);
		System.out.println(resultado.getValor());
		viagem.imprimeHorarios(resultado.getSolucao());
		
		System.out.println("\nOtimizacao genetica:");
		resultado = otimizador.otimizacaoGenetica(dominio, viagem);
		System.out.println(resultado.getValor());
		viagem.imprimeHorarios(resultado.getSolucao());
		
		System.out.println("\nOtimizacao gulosa:");
		resultado = otimizador.otimizacaoGulosa(dominio, viagem);
		System.out.println(resultado.getValor());
		viagem.imprimeHorarios(resultado.getSolucao());
	}
}
